
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <EEPROM.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
// The pins for I2C are defined by the Wire-library. 
// On an arduino UNO:       A4(SDA), A5(SCL)
// On an arduino MEGA 2560: 20(SDA), 21(SCL)
// On an arduino LEONARDO:   2(SDA),  3(SCL), ...
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3C ///< See datasheet for Address; 0x3D for 128x64, 0x3C for 128x32
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);


//Constants:
const int relayPin = 12;  //relays control

const int setPin = 7; 
//boolean set = false; 
int set=0;
const int startPin = 3;  
int start=0;

const int potPin = A0; //pin A0 to read potentiometer analog input, brake voltage
const int voltPin = A1; //pin 
const int ampPin = A2; //pin 

//Variables:
float value; //save potentiometer analog value
float volts=0.00;
float tempv=0.00;

float amps=0.000;
float mAmps=0.000;
float tempa=0.000;
float r1=30000.0;    //r1 value 1megom
float r2=4300.0;      //r2 value 20k gnd. Adjust for corrections.
float minV=2.7;
float maxV=4.2;
float power=0.0;
float maSec=0.000;
float maMin=0.000;
float mah=0.000;
float waSec=0.000;
float waMin=0.000;
float wah=0.000;
float wh=0.000;
int relay=0;
int timeS=0; //in sec
int timeM=0; //in min
int timeH=0; //in hours

int status=0; //0-off 1-start 2-end
int setStatus=0; //0-nothing 1-set min volts 2-set max volts

const unsigned long measureVoltInterval = 125; //
unsigned long previousMeasureVoltTime = 0;

const unsigned long measureAmpInterval = 175; //
unsigned long previousMeasureAmpTime = 0;

const unsigned long minmaxInterval = 185; //ms 
unsigned long previousMinmaxTime = 0;

const unsigned long displayInterval = 100; //ms 
unsigned long previousDisplayTime = 0;

const unsigned long buttonsInterval = 225; //ms 
unsigned long previousButtonsTime = 0;

const unsigned long MahInterval = 1000; //ms =1s
unsigned long previousMahTime = 0;

const unsigned long DisplayChangeInterval = 4000; //ms =1s
unsigned long previousDisplayChange = 0;
boolean DisplayCh=false;



void setup() {
  // put your setup code here, to run once:
 Serial.begin(9600);
 pinMode(relayPin, OUTPUT);
 pinMode(potPin, INPUT); //Optional
 pinMode(voltPin, INPUT); //Optional
  pinMode(ampPin, INPUT); //Optional 

  pinMode(setPin, INPUT_PULLUP);
  pinMode(startPin, INPUT_PULLUP);

minV=EEPROM.get(0,minV);
maxV=EEPROM.get(100,maxV);

   // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("SSD1306 OLED allocation failed"));
    //for(;;); // Don't proceed, loop forever
  }
 
  display.clearDisplay();
  display.setTextSize(1,2);      // Normal 1:1 pixel scale
  display.setTextColor(SSD1306_WHITE); // Draw white text
      display.setCursor(0,0);
      display.println("Lithium");
      display.setCursor(0,16); // 8symbols x5=40
      display.println("Battery");
      display.setCursor(0,32); // 8symbols x5=40
      display.println("Tester v2");
      display.setCursor(0,48); // 8symbols x5=40
      display.println("~10A 1V-40V");
  display.display();
   delay(5000);
    // Clear the buffer
  display.clearDisplay();
}

void loop() {
  // put your main code here, to run repeatedly:
  unsigned long currentTime = millis();




 // check if Start pushbutton is pressed. If it is, the buttonState is LOW:---
 if (currentTime -  previousButtonsTime >= buttonsInterval) {
   if (digitalRead(startPin) == LOW){
   //start = !start;
   start=start+1;
   delay(10);
   //Serial.println(start);
     if (start==1 && status==0) {   status=1;  }
  if (start==2 && status==1) {   status=0; start=0; }
  if ((start==1 || start==2) && status==2) {   status=0; start=0; }
   }

   if (digitalRead(setPin) == LOW){
   //set = !set;
   set=set+1;
   delay(10);
     if (set==1 && setStatus==0) {   setStatus=1;  }
  if (set==2 && setStatus==1) {   setStatus=2;    }
  if (set==3 && setStatus==2) {  setStatus=0; set=0;  
   EEPROM.put(0, minV); 
   EEPROM.put(100, maxV);
     minV=EEPROM.get(0,minV);
     maxV=EEPROM.get(100,maxV);
   }
    // if (set==4 && setStatus==3) {    setStatus=0; set=0; 
    //  minV=EEPROM.get(1,minV);
    //  maxV=EEPROM.get(2,maxV);
    // }
}

 previousButtonsTime = currentTime;
 }
//-----------------------------------------------------------------------------


//---------------set min max voltages------------------------------------------
if (setStatus==1) {
 // delay(10);
//read voltage potentionemeter
  value = analogRead(potPin);          //Read and save analog value from potentiometer
  value = map(value, 0, 1023, 0, 400)/10.0; //Map value 0-1023 to 0-255 (PWM) max 60v. /10.0 needed to get decimal points

  minV=value;
delay(100);
}
if (setStatus==2) {
  value = analogRead(potPin);          //Read and save analog value from potentiometer
  value = map(value, 0, 1023, 0, 400)/10.0; //Map value 0-1023 to 0-255 (PWM) max 60v
  maxV=value;
delay(100);
}
if (setStatus==3) {
delay(100);
}
//----------------------------------------------------------------------------------



//---call read voltage and current functions -----------------------------
if ((currentTime -  previousMeasureVoltTime >= measureVoltInterval) && setStatus==0) {
voltageRead();
previousMeasureVoltTime = currentTime;
}

if ((currentTime -  previousMeasureAmpTime >= measureAmpInterval) && setStatus==0) {
ampsRead();
previousMeasureAmpTime = currentTime;
}
//--------------------------------------------------------------------

//------count mah-----------------------------------------------------
if ((currentTime -  previousMahTime >= MahInterval) && setStatus==0) {
power=volts*amps;

if(power>0 && status==1){
mAmps=amps*1000; //1A =1000mA
maSec+=mAmps;
maMin=maSec/60;
mah=maMin/60;

waSec+=power;
waMin=waSec/60;
wh=waMin/60;
/*
Serial.print("amps: ");
 Serial.println(amps,4);
 Serial.print("maSec: ");
 Serial.println(maSec,4);
  Serial.print("maMin: ");
 Serial.println(maMin,4);
  Serial.print("mah: ");
 Serial.println(mah,4);
 */
//1 Amperes Hour= 60 Amperes Minute
timeS+=1;
if(timeS>=60){timeM+=1;timeS=0;}
if(timeM>=60){timeH+=1;timeM=0;}
}
previousMahTime = currentTime;
}
//------------------------------------------------------------------

 //-------check min max-----------------------------------------------
if ((currentTime -  previousMinmaxTime >= minmaxInterval) && setStatus==0 && status==1) {
if( volts<minV){
  status=2;
}
if( volts>maxV){
  status=2;
}
previousMinmaxTime = currentTime;
}
//------------------------------------------------------------------------



//----relay control--------------
 if(status==1 && setStatus==0){
   digitalWrite(relayPin, HIGH); //relays off
   //voltageRead();
 }
  if(status==0 && setStatus==0){
   digitalWrite(relayPin, LOW); //relays off
   //voltageRead();
 }
   if(status==2 && setStatus==0){
   digitalWrite(relayPin, LOW); //relays off
   //voltageRead();
 }
 //------------------------------------------

//---------------Display change-----------------------
if (currentTime -  previousDisplayChange >= DisplayChangeInterval) {
  DisplayCh=!DisplayCh; 
////Serial.print("Eprom1: ");
// Serial.println(EEPROM.get(1,minV),4);
//Serial.print("Eprom2: ");
// Serial.println(EEPROM.get(2,maxV),4);

  previousDisplayChange=currentTime;
}
//-------------------------------------------------------

//------------------------------------------------
  
if (currentTime -  previousDisplayTime >= displayInterval) {  
 // delay(10);
 display.clearDisplay();
   display.setTextSize(1,2);      // Normal 1:1 pixel scale
   display.setTextColor(SSD1306_WHITE); // Draw white text
      
      if(setStatus==0){
      display.setCursor(0,0);
      display.println("Volts: ");
      display.setCursor(40,0); // 8symbols x5=40
      display.println(volts,2);
    display.setCursor(70,0);
    display.println("Amp: ");    
    display.setCursor(95,0);
    display.println(amps,2);  
    
   display.setCursor(0,16);
   display.println("minV: ");
   display.setCursor(40,16); // 9symbols x5=45
   display.println(minV,1);

   display.setCursor(70,16);
   display.println("maxV: ");
   display.setCursor(97,16); // 9symbols x5=45
   display.println(maxV,1);

//power=1000;
if(DisplayCh){
   display.setCursor(0,32);
   display.println("Pw: ");
   display.setCursor(20,32); // 9symbols x5=45
   display.println(power,2);
}
else{
   display.setCursor(0,32);
   display.println("Wh: ");
   display.setCursor(20,32); // 9symbols x5=45
   display.println(wh,1);
}

   display.setCursor(60,32);
   display.println("mAh: ");
   display.setCursor(83,32); // 9symbols x5=45
   display.println(mah,0);
   //display.println("100000");

   display.setCursor(0,48);
   display.println("Stat: ");
   display.setCursor(32,48); // 9symbols x5=45
   if(status==0){display.println("Off");}
   if(status==1){display.println("On");}
   if(status==2){display.println("End");}

   display.setCursor(54,48);
   display.println("Time");
   display.setCursor(80,48); // 9symbols x5=45
   display.print(timeH);
   display.print(":");
   display.print(timeM);
   display.print(":");
   display.print(timeS);
   }
   if(setStatus==1){
   display.setCursor(0,16);
   display.println("Min voltage: ");
   display.setCursor(90,16); // 8symbols x5=40
   display.println(minV,2);
   }
   if(setStatus==2){
   display.setCursor(0,16);
   display.println("Max voltage: ");
   display.setCursor(90,16); // 8symbols x5=40
   display.println(maxV,2);
   }

 
       display.display();
previousDisplayTime=currentTime;
}//end of display 

  
} //end of loop


void voltageRead(){
  volts=0.00;
  float analogvalue=0.00;
  for(int i=0; i<50; i++){
      analogvalue += analogRead(voltPin);
     // delay (1); // let ADC settle before next sample 1ms
  }
     analogvalue=analogvalue/50;
      tempv = (analogvalue * 5.00) / 1024.00;  // FORMULA USED TO CONVERT THE VOLTAGE
     volts = tempv ;
    volts = tempv / (r2/(r1+r2));
}

void ampsRead(){
  amps=0;
  float analogvalue=0;
  for(int i=0; i<50; i++){
      analogvalue += analogRead(ampPin);
      // delay (1); // let ADC settle before next sample 1ms
  }
     analogvalue=analogvalue/50;
     if(analogvalue<0){analogvalue=analogvalue*-1;}
     tempa = (2.5 - (analogvalue * (5.0 / 1024.0)) )/0.100; //20A-0.100;
     amps = tempa ; 
     
     amps=amps-0.32; //stupid mistake I placed magnetic relay to close to ACS712 hall effect current sensor
     if (amps<0){amps=amps*-1;} //if using as charger
     if (amps<0.28){amps=0;}

}
